/*** NetAtmo Z-Way HA module *******************************************

Version: 1.1.3
(c) Z-Wave.Me, 2015
-----------------------------------------------------------------------------
Author: Lucas von Hacht <lh@zwave.eu>
Modified: Poltorak Serguei <ps@z-wave.me>, Niels Roche <nir@zwave.eu>
Description:
    This module creates a weather widget

******************************************************************************/

// ----------------------------------------------------------------------------
// --- Class definition, inheritance and setup
// ----------------------------------------------------------------------------

function NetAtmo (id, controller) {
    // Call superconstructor first (AutomationModule)
    NetAtmo.super_.call(this, id, controller);
}

inherits(NetAtmo, AutomationModule);

_module = NetAtmo;

// ----------------------------------------------------------------------------
// --- Module instance initialized
// ----------------------------------------------------------------------------

NetAtmo.prototype.init = function (config) {
    NetAtmo.super_.prototype.init.call(this, config);

    var self = this;
    
    this.vDev = null;

    this.timer = setInterval(function() {
        self.refreshToken();
    }, 3540*1000);
    
    this.authenticate();
};

NetAtmo.prototype.stop = function () {
    NetAtmo.super_.prototype.stop.call(this);

    if (this.timer)
    {
      clearInterval(this.timer);
    }

    var filterId = "NetAtmo_" + this.id + "_",
    filterArray = this.controller.devices.filter(function(device){
      return device.id.indexOf(filterId) > -1;
    });

    filterArray.forEach(function(device){
      this.controller.devices.remove(device.id);
    });
};

// ----------------------------------------------------------------------------
// --- Module methods
// ----------------------------------------------------------------------------

NetAtmo.prototype.createDevice = function (probe, scale, level, icon, name, number, deviceId, moduleId) {
	//Gerät erzeugen
  var self = this,
      moduleName = "NetAtmo",
      langFile = self.controller.loadModuleLang(moduleName);

	this.controller.devices.create({
	        deviceId: "NetAtmo_" + this.id + "_" + number,
	        defaults: {
	            deviceType: 'sensorMultilevel',
	            metrics: {
	            	probeTitle: probe,
					      scaleTitle: scale,
					      level: level,
				      	icon: icon,
				      	title: probe
	            }
	        },
	        overlay: {
	            metrics: {
	                scaleTitle: scale,
	                title: name + " " + probe
	            }
	        },
          handler: function (command)
          {
            if(command === "update")
            {
              var url;

              if(moduleId === null)
                url = "https://api.netatmo.net/api/getmeasure?access_token="+self.config.accessToken+"&device_id="+deviceId+"&scale=3hours&type="+probe+"&date_end=last";
              else
                url = "https://api.netatmo.net/api/getmeasure?access_token="+self.config.accessToken+"&device_id="+deviceId+"&module_id="+moduleId+"&scale=3hours&type="+probe+"&date_end=last";

              http.request ({
                url: url,
                method: "GET",
                async: true,
                success: function(rsp){
                  var filterId = "NetAtmo_" + self.id + "_" + number, filterArray = self.controller.devices.filter(function(device){
                  return device.id.indexOf(filterId) > -1;
                  });

                  var valueArray = rsp.data.body[0].value[0];
                  filterArray[0].set("metrics:level", valueArray[0]);
                },
                error: function(rsp){
                  self.controller.addNotification("error", langFile.update + name +" "+ probe, langFile.connection, "NetAtmo");
                  console.log("Update fehlgeschlagen");
                },
                complete: function(rsp){console.log("Update beendet");}
              });
            }
          },
	        moduleId: this.id
	   });
};

NetAtmo.prototype.fetchWeather = function() {
	var url = "https://api.netatmo.net/api/devicelist?access_token="+this.config.accessToken,
      moduleName = "NetAtmo",
      self = this,
      langFile = self.controller.loadModuleLang(moduleName);

  http.request ({
      url: url,
      method: "GET",
      async: true,
      success: function(rsp){
        //console.log(JSON.stringify(rsp, null, 4));
        var deviceLength = rsp.data.body.devices.length, moduleLength = rsp.data.body.modules.length;
        for(i=0; i < deviceLength; i++)
        {
            var device = rsp.data.body.devices[i],
            pressure = device.dashboard_data.Pressure,
            co2 = device.dashboard_data.CO2,
            humidity = device.dashboard_data.Humidity,
            noise = device.dashboard_data.Noise,
            temperature = device.dashboard_data.Temperature,
            updateId = device._id,
            name = device.station_name;
                    
            //Geräte mit Werten erzeugen
            for(k=0; k<5; k++)
            {
                switch(k) {
                  case 0:
                    //temepratur erzeugen
                    self.createDevice('Temperature', 'ºC', temperature, 'temperature', name, "WeatherDevice_" + i +"_"+k, updateId, null);
                    break;
                          
                  case 1:
                    //pressure erzeugen
                    self.createDevice('Pressure', 'mb', pressure, 'multilevel', name, "WeatherDevice_" + i +"_"+k, updateId, null);
                    break;
                          
                  case 2:
                    //co2 erzeugen
                    self.createDevice('CO2', 'ppm', co2, 'co', name, "WeatherDevice_" + i +"_"+k, updateId, null);
                    break;
                          
                  case 3:
                    //humidity erzeugen
                    self.createDevice('Humidity', '%', humidity, 'humidity', name, "WeatherDevice_" + i +"_"+k, updateId, null);
                    break;
                          
                  case 4:
                    //noise erzeugen
                    self.createDevice('Noise', 'db', noise, 'multilevel', name, "WeatherDevice_" + i +"_"+k, updateId, null);
                    break;
                          
                  default:
                    break;
                }
            }
        }
                
        for(j=0; j < moduleLength; j++)
        {
            var module = rsp.data.body.modules[j];

            if(module.type === ("NAModule1" || "NAModule4"))
            {
              humidity_module = module.dashboard_data.Humidity;
              temperature_module = module.dashboard_data.Temperature;
              updateId = module._id;
              deviceId = module.main_device;
              name_module = module.module_name;
                    
              //Geräte erzeugen
              for(k=0; k < 2; k++)
              {
                switch(k) {
                  case 0:
                    //temepratur erzeugen
                    self.createDevice('Temperature', 'ºC', temperature_module, 'temperature', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                    break;
                          
                  case 1:
                    //humidity erzeugen
                    self.createDevice('Humidity', '%', humidity_module, 'humidity', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                    break;
                             
                  default:
                    break;
                }
              }
            }
            else if(module.type === "NAModule3")
            {
              var hourRain = module.dashboard_data.sum_rain_1,
                  dayRain = module.dashboard_data.sum_rain_24,
                  updateId = module._id,
                  deviceId = module.main_device,
                  name_module = module.module_name;

              for(k=0; k < 2; k++)
              {
                switch(k) {
                  case 0:
                    self.createDevice('Rain', 'mm', hourRain, 'humidity', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                    break;
                          
                  case 1:
                    self.createDevice('Sum_Rain', 'mm', dayRain, 'humidity', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                    break;
                             
                  default:
                    break;
                }
              }
            }
            else if(module.type === "NAModule2")
            {
              var windAngle = module.dashboard_data.WindAngle,
                  windStrength = module.dashboard_data.WindStrength,
                  gustStrength = module.dashboard_data.GustStrength,
                  updateId = module._id,
                  deviceId = module.main_device,
                  name_module = module.module_name;

                for(k=0; k < 3; k++)
                {
                  switch(k) {
                    case 0:
                      //Windrichtung erzeugen
                      self.createDevice('WindAngle', '°', windAngle, 'multilevel', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                      break;
                          
                    case 1:
                      //Windstärke erzeugen
                      self.createDevice('WindStrength', 'km/h', windStrength, 'multilevel', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                      break;

                    case 2:
                      //Stärksten Wind erzeugen
                      self.createDevice('GustStrength', 'km/h', gustStrength, 'multilevel', name_module, "WeatherModule_" + j +"_"+k, deviceId, updateId);
                             
                    default:
                      break;
                  }
                }
            }
        }
      },
      error: function(rsp){
        console.log("Wetter konnte nicht abgerufen werden");
        self.controller.addNotification("error", langFile.errWeather, langFile.connection, "NetAtmo");
      },
      complete: function(rsp){console.log("Wetter Request beendet");}
      });
};

NetAtmo.prototype.fetchThermostat = function() {
	
	var url = "https://api.netatmo.net/api/devicelist?access_token="+this.config.accessToken+"&app_type=app_thermostat",
      moduleName = "NetAtmo",
      self = this,
      langFile = self.controller.loadModuleLang(moduleName);
  
  http.request ({
      url: url,
      method: "GET",
      async: true,
      success: function(rsp){
        //console.log(JSON.stringify(rsp, null, 4));
        var deviceLength = rsp.data.body.devices.length, moduleLength = rsp.data.body.modules.length;
        for(i=0; i<deviceLength; i++)
        {
            var device = rsp.data.body.devices[i],
            name = device.station_name,
            updateId = device._id,
            temperature = rsp.data.body.modules[i].dashboard_data.Temperature;
                    
            //Geräte mit werten erzeugen
            var vDev = self.controller.devices.create({
              deviceId: "NetAtmo_"+self.id+"_Thermostat_"+i,
              defaults: {
                deviceType: "thermostat",
                metrics: {
                  scaleTitle: "ºC",
                  level: temperature,
                  min: 7,
                  max: 30,
                  icon: 'thermostat',
                  title: "Thermostat "+name
                }
            },
            overlay: {},
            handler: function(command, args) {
              if(command === "exact")
              {
                  url = "https://api.netatmo.net/api/setthermpoint?access_token="+ self.config.accessToken +"&device_id="+ device._id +"&module_id="+ device.modules[0] +"&setpoint_mode=manual&setpoint_temp="+args;

                  http.request ({
                    url: url,
                    method: "GET",
                    async: true,
                    success: function(rsp){
                      console.log("Thermpoint set");
                    },
                    error: function(rsp){
                      console.log("Thermpoint fehlgeschlagen");
                      self.controller.addNotification("error", langFile.errSetTherm, langFile.connection, "NetAtmo");
                    },
                    complete: function(rsp){console.log("Thermpoint request done");}
                  });
              }
            },
            moduleId: self.id
          });
        }

        for(j=0; j < moduleLength; j++)
        {
          var module = rsp.data.body.modules[j],
          temperature = module.dashboard_data.Temperature,
          updateId = module._id,
          deviceId = module.main_device,
          name = module.module_name;

          self.createDevice('Temperature', 'ºC', temperature, 'temperature', name, "ThermModule_" + j, deviceId, updateId);
        }

      },
      error: function(rsp){
        console.log("Thermostat konnte nicht abgerufen werden");
        self.controller.addNotification("error", langFile.errTherm, langFile.connection, "NetAtmo");
      },
      complete: function(rsp){console.log("Thermostat Request beendet");}
    });
};


NetAtmo.prototype.refreshToken = function() {
    var self = this;
    var app_id = self.config.app_id, 
      app_secret = self.config.app_secret,
      moduleName = "NetAtmo",
      langFile = self.controller.loadModuleLang(moduleName),
    	url = "https://api.netatmo.net/oauth2/token";

    http.request ({
      url: url,
      method: "POST",
      headers:{
        "Content-type" : "application/x-www-form-urlencoded;charset=UTF-8"
      },
      data:{
        "grant_type": "refresh_token",
        "refresh_token": self.config.refreshToken,
        "client_id": app_id,
        "client_secret": app_secret
      },
      async: true,
      success: function(rsp){
        self.config.accessToken = rsp.data.access_token; 
        self.config.refreshToken = rsp.data.refresh_token;
      },
      error: function(rsp){
        console.log("Auth fehlgeschlagen");
        self.controller.addNotification("error", langFile.errRefresh, langFile.connection, "NetAtmo");
      },
      complete: function(rsp){console.log("Finished refreshing");}
      });
};

NetAtmo.prototype.authenticate = function() {
    var self = this;
    var app_id = self.config.app_id,
      app_secret = self.config.app_secret,
      moduleName = "NetAtmo",
      langFile = self.controller.loadModuleLang(moduleName),
    	scope = "read_station read_thermostat write_thermostat",
    	url = "https://api.netatmo.net/oauth2/token";
         
    http.request ({
      url: url,
      method: "POST",
      headers:{
        "Content-type" : "application/x-www-form-urlencoded;charset=UTF-8"
      },
      data:{
        "grant_type": "password",
        "client_id": app_id,
        "client_secret": app_secret,
        "username": self.config.user,
        "password": self.config.password,
        "scope": scope
      },
      async: true,
      success: function(rsp){
        self.config.accessToken = rsp.data.access_token; 
        self.config.refreshToken = rsp.data.refresh_token; 
        self.fetchWeather(); 
        self.fetchThermostat();
      },
      error: function(rsp){
        console.log("Auth fehlgeschlagen");
        self.controller.addNotification("error", langFile.errAuth, langFile.connection, "NetAtmo");
    },
      complete: function(rsp){console.log("finished auth");}
      });
};
